@@ -421,6 +421,17 @@ datastep <- function(data, steps, keep = NULL,
   # Capture number of starting columns
   startcols <- ncol(data)
   
+  # Put code in a variable for safe-keeping
+  code <- substitute(steps, env = environment())
+  
+  # Put aggregate functions in a variable 
+  agg <- substitute(calculate, env = environment())
+  
+  # Execute aggregate functions
+  if (paste0(deparse(agg), collapse = "") != "NULL") {
+    data <- within(data, eval(agg), keepAttrs = TRUE)
+  }
+  
   # Apply variable attributes
   if (!is.null(attrib)) {
     for (nm in names(attrib)) { 
@@ -458,15 +469,6 @@ datastep <- function(data, steps, keep = NULL,
     }
   }
   
-  # Put code in a variable for safe-keeping
-  code <- substitute(steps, env = environment())
-  
-  # Put aggregate functions in a variable 
-  agg <- substitute(calculate, env = environment())
-  if (paste0(deparse(agg), collapse = "") != "NULL") {
-    data <- within(data, eval(agg), keepAttrs = TRUE)
-  }
-  
   ret <- list()
   firstval <- NULL
   firstvals <- list()
@@ -493,7 +495,7 @@ datastep <- function(data, steps, keep = NULL,
     # Deal with 1 column situation
     data_attributes <- data[1, ]
   } else {
-    data_attributes <- data.frame(data[1, ])
+    data_attributes <- data.frame(data[1, ], stringsAsFactors = FALSE)
     names(data_attributes) <- names(data)
   }
   # Tibble subset will keep attributes, but data.frame will not
@@ -506,11 +508,11 @@ datastep <- function(data, steps, keep = NULL,
   # Temporarily convert to a data frame.  
   # Seriously like 20X performance increase.
   if (any("grouped_df" == class(data)))
-    data <- as.data.frame(data)
+    data <- as.data.frame(data, stringsAsFactors = FALSE)
   
   # data.table is not that bad, but data.frame is better.
   if (any("data.table" == class(data)))
-    data <- as.data.frame(data)
+    data <- as.data.frame(data, stringsAsFactors = FALSE)
   
   # Add automatic variables
   data <- add_autos(data, by, sort_check)
@@ -522,7 +524,7 @@ datastep <- function(data, steps, keep = NULL,
     if (ncol(data) > 1)
       rw <- data[n., ]
     else {
-      rw <- data.frame(data[n., ])
+      rw <- data.frame(data[n., ], stringsAsFactors = FALSE)
       names(rw) <- names(data)
     }
     
@@ -622,281 +624,9 @@ datastep <- function(data, steps, keep = NULL,
 
 
 
-#' @noRd
-datastep_back <- function(data, steps, keep = NULL,
-                     drop = NULL, rename = NULL,
-                     by = NULL, calculate = NULL,
-                     retain = NULL,
-                     sort_check = TRUE) {
-  
-  if (!"data.frame" %in% class(data))
-    stop("input data must be inherited from data.frame")
-  
-  
-  if (!is.null(retain)) {
-    if (!"list" %in% class(retain))
-      stop("retain parameter value must be of class 'list'")
-    
-  }
-  
-  # Capture number of starting columns
-  startcols <- ncol(data)
-  
-  # Put code in a variable for safe-keeping
-  code <- substitute(steps, env = environment())
-  
-  # Put aggregate functions in a variable 
-  agg <- substitute(calculate, env = environment())
-  if (paste0(deparse(agg), collapse = "") != "NULL") {
-    data <- within(data, eval(agg), keepAttrs = TRUE)
-  }
-  
-  ret <- list()
-  firstval <- NULL
-  firstvals <- list()
-  rowcount <- nrow(data)
-  orig_class <- class(data)
-  
-  # Set by if data is a grouped tibble
-  if (is.null(by) && "grouped_df" %in% class(data)) {
-    if (!is.null(attr(data, "groups"))) {
-      grpdf <- attr(data, "groups")
-      nms <- names(grpdf)
-      if (!is.null(nms)) {
-        nms <- nms[nms != ".rows"]
-        if (length(nms) > 0) {
-          by <- nms
-          
-        }
-      }
-    }
-  }
-  
-  # Deal with 1 column situation
-  if (ncol(data) > 1) {
-    data_attributes <- data[1, ]
-  } else {
-    data_attributes <- data.frame(data[1, ])
-    names(data_attributes) <- names(data)
-  }
-  
-  
-  # For some reason the grouped tibble kills performance.
-  # Temporarily convert to a data frame.  
-  # Seriously like 20X performance increase.
-  if (all("data.frame" != class(data)))
-    data <- as.data.frame(data)
-
-  
-  # Increases performance
-  if (!is.null(by)) {
-    bydata <- as.data.frame(data[ , by])
-  }
-  
-  
-  # Step through row by row
-  for (n. in seq_len(rowcount)) {
-    
-    # If one column, subset comes back with a vector
-    if (ncol(data) > 1)
-      rw <- data[n., ]
-    else {
-      rw <- data.frame(data[n., ])
-      names(rw) <- names(data)
-    }
-    
-    if (!is.null(by))
-      byrw <- rw[1, by]
-    
-    # Deal with first. and last.
-    # These can be accessed from within the evaluated code,
-    # which is really cool.
-    if (!is.null(by)) {
-      if (is.null(firstval)) {
-        firstval <- byrw
-        if (length(names(firstval)) == 0)
-          names(firstval) <- by
-        first. <- TRUE
-      } else {
-        
-        # Compare current by group to previous row
-        if (dfcomp(firstval, byrw) == FALSE) {
-          first. <- TRUE
-          firstval <- byrw
-          if (length(names(firstval)) == 0)
-            names(firstval) <- by
-        } else {
-          first. <- FALSE
-        }
-      }
-      
-      # If it's the last row of the data frame, mark last.
-      if (n. == rowcount) {
-        last. <- TRUE
-      } else {
-        
-        # Compare by group to next row to determine last.
-        # print(bydata)
-        # print(n.)
-        # print(bydata[n. + 1, ])
-        # print(byrw)
-        if (dfcomp(bydata[n. + 1, ],  byrw) == FALSE) {
-          last. <- TRUE
-        } else {
-          last. <- FALSE
-        }
-      }
-      
-    } else {
-      
-      # If no by group is specified, mark the first and last rows
-      # of the entire data frame
-      
-      # If it's the first row
-      if (n. == 1)
-        first. <- TRUE
-      else
-        first. <- FALSE
-      
-      # If it's the last row
-      if (n. == rowcount)
-        last. <- TRUE
-      else
-        last. <- FALSE
-      
-    }
-    
-    
-    # Deal with retained variables
-    if (!is.null(retain)) {
-      if (length(ret) == 0) {
-        for (nm in names(retain)) {
-          
-          # Populate with initial value
-          rw[[nm]] <- retain[[nm]]
-          
-        }
-        
-      } else {
-        for (nm in names(retain)) {
-          
-          # Populate with value from previous row   
-          #data[n., nm] <- ret[n. - 1, nm]  way backup
-          
-          rw[[nm]] <- ret[[n. - 1]][[nm]] # current
-          
-          
-        }
-      }
-    }
-    
-    # Evaluate the code for the row
-    ret[[n.]]  <- within(rw, eval(code), keepAttrs = TRUE)
-    
-    
-    # Keep track of the groups
-    if (!is.null(by) & first. & sort_check) {
-      firstvals[[length(firstvals) + 1]] <- firstval
-    }
-  }
-  
-  # Bind all rows
-  ret <- bind_rows(ret, .id = "column_label")
-  ret["column_label"] <- NULL
-  
-  if (sort_check & !is.null(by)) {
-    if (length(firstvals) > 0) {
-      d <- bind_rows(firstvals, .id = "column_label")
-      d["column_label"] <- NULL
-      ddat <- distinct(d)
-      if (nrow(ddat) != nrow(d)) {
-        stop(paste("Input data is not sorted according to the 'by' variable",
-                   "parameter.\n  Either sort the input data properly or",
-                   "set the sort_check parameter to FALSE."))
-      }
-    }
-  }
-  
-  # Perform drop operation
-  if (!is.null(drop))
-    ret <- ret[ , !names(ret) %in% drop]
-  
-  # Perform keep operation
-  if (!is.null(keep)) {
-    ret <- ret[ , keep]
-  }
-  
-  
-  # Convert back to tibble if original was a tibble
-  if ("tbl_df" %in% orig_class & !"tbl_df" %in% class(ret)) {
-    ret <- as_tibble(ret)
-  }
-  
-  # Put back grouping attributes if original data was grouped
-  if (!is.null(by) & "grouped_df" %in% orig_class) {
-    
-    if (all(by %in% names(ret)))
-      ret <- group_by(ret, across({{by}})) 
-    
-  }
-  
-  # Restore attributes from original data 
-  ret <- copy_attributes(data_attributes, ret)
-  
-  
-  # Perform rename operation
-  if (!is.null(rename)) {
-    nms <- names(ret)
-    names(ret) <- ifelse(nms %in% names(rename), rename, nms)
-  }
-  
-  endcols <- ncol(ret)
-  if (startcols > endcols)
-    log_logr(paste0("datastep: columns increased from ", startcols, " to ", 
-                    endcols))
-  else if (startcols < endcols)
-    log_logr(paste0("datastep: columns decreased from ", startcols, " to ", 
-                    endcols))
-  else 
-    log_logr(paste0("datastep: columns started with ", startcols, 
-                    " and ended with ", endcols))
-  
-  return(ret)
-}
-
-
-
 # Utilities ---------------------------------------------------------------
 
 
-#' @description A function to compare two by groups, passed as data frames
-#' @noRd
-dfcomp <- function(df1, df2) {
-  names(df1) <- NULL
-  names(df2) <- NULL
-  ret <- FALSE
-
-  if (all(is.null(df1)) && all(is.null(df2)))
-    ret <- TRUE
-  else if (all(is.na(df1)) && all(is.na(df2)))
-    ret <- TRUE
-  if (all(is.data.frame(df1) && is.data.frame(df2))) {
-
-    for (i in seq_along(df1)) {
-      if (any(df1[[i]] != df2[[i]]))
-        return(FALSE)
-    }
-    ret <- TRUE
-
-  } else if (all(class(df1) == class(df1))) {
-
-    ret <- all(df1 == df2)
-
-  }
-  
-  return(ret)
-}
-
 
 #' @noRd
 copy_attributes <- function(df1, df2) {
